const char rcsid_kvcv_c[] = "$Id: kvcv.c,v 1.3 1996/12/11 19:16:46 marc Exp $";

/* 
 * Copyright (c) 1996, Marc Horowitz.  All rights reserved.
 * See the LICENSE file in the release for redistribution information.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "pgputil.h"
#include "md5.h"
#include "pgpfile.h"

/* when you see a type 6 (pubkey), forget all state
   when you see the first type 13, print the pk header
   */

struct state {
   int got_pk, got_userid, got_revoke;
   ddesc pubkey;
   long pubkeylen;
};

int kvcv(ddesc *packet, void *c)
{
   long ptype, plen;
   struct state *s = (struct state *) c;

   if (!decode_psf(packet, &ptype, &plen))
      return(0);

   switch (ptype) {
    case 6:
      /* public key packet */
      {
	 s->got_pk = 1;
	 s->got_userid = 0;
	 s->got_revoke = 0;
	 s->pubkey = *packet;
	 s->pubkeylen = plen;
      }
      break;
    case 13:
      /* user id packet */
      {
	 int i;
	 ddesc userid;

	 if (!decode_userid(packet, plen, &userid))
	    return(0);

	 if (s->got_userid) {
	    printf("                              %.*s\n",
		   (int) userid.size, userid.data);
	 } else if (s->got_pk) {
	    mpidesc modulus, exponent;
	    time_t create_time;
	    MD5_CTX md5ctx;
	    unsigned char hash[16];
	    struct tm *c_tm;

	    if (!decode_pubkey(&(s->pubkey), s->pubkeylen, &modulus,
			       &exponent, &create_time))
	       return(0);

	    MD5Init(&md5ctx);
	    MD5Update(&md5ctx, modulus.number.data, modulus.number.size);
	    MD5Update(&md5ctx, exponent.number.data, exponent.number.size);
	    MD5Final(hash, &md5ctx);

	    /* pgp does gmtime, so we do, too */
	    c_tm = gmtime(&create_time);

	    printf("pub%6d/%02X%02X%02X%02X %04d/%02d/%02d %s%.*s\n"
		   "          Key fingerprint =  ",
		   (int) modulus.nbits,
		   modulus.number.data[modulus.number.size-4],
		   modulus.number.data[modulus.number.size-3],
		   modulus.number.data[modulus.number.size-2],
		   modulus.number.data[modulus.number.size-1],
		   c_tm->tm_year+1900, c_tm->tm_mon+1, c_tm->tm_mday,
		   (s->got_revoke?
		    "*** KEY REVOKED ***\n                              ":""),
		   (int) userid.size, userid.data);

	    for (i=0; i<8; i++)
	       printf("%02X ", hash[i]);
	    printf(" ");
	    for (i=8; i<16; i++)
	       printf("%02X ", hash[i]);
	    printf("\n");

	    s->got_userid = 1;
	 }
      }
      break;
    case 2:
      /* signature packet */
      {
	 ddesc keyid;
	 long sigclass;
	 time_t sigtime;
	 int i;

	 if (!decode_sig(packet, plen, &keyid, &sigclass, &sigtime))
	    return(0);

	 if (sigclass == 0x20) {
	    /* key revoked */
	    s->got_revoke = 1;
	 } else if (sigclass == 0x10) {
	    printf("sig       ");
	    for (i=4; i<8; i++)
	       printf("%02X", keyid.data[i]);
	    printf("             (can't do uid->name conversion yet)\n");
	 } else {
	    printf("             (funny signature packet (class %02lX))\n",
		   sigclass);
	 }
      }
      break;
   }

   return(1);
}

int do_file(ddesc *data)
{
   struct state s;

   s.got_pk = 0;
   s.got_userid = 0;
   s.got_revoke = 0;

   return(decode_file(data, kvcv, (void *) &s));
}
